@@ -2,7 +2,6 @@ import argparse
 import json
 import os
 import sys
-import time
 from collections import defaultdict
 
 try:
@@ -18,7 +17,7 @@ try:
 except ImportError:
     pygments = None
 
-from .client import read_config, CloudStack, CloudStackException  # noqa
+from .client import read_config, CloudStack, CloudStackApiException, CloudStackException  # noqa
 
 
 __all__ = ['read_config', 'CloudStack', 'CloudStackException']
@@ -45,9 +44,9 @@ def _format_json(data, theme):
     return output
 
 
-def main():
+def main(args=None):
     parser = argparse.ArgumentParser(description='Cloustack client.')
-    parser.add_argument('--region', metavar='REGION',
+    parser.add_argument('--region', '-r', metavar='REGION',
                         help='Cloudstack region in ~/.cloudstack.ini',
                         default=os.environ.get('CLOUDSTACK_REGION',
                                                'cloudstack'))
@@ -61,6 +60,9 @@ def main():
                         help='do not wait for async result')
     parser.add_argument('--quiet', '-q', action='store_true', default=False,
                         help='do not display additional status messages')
+    parser.add_argument('--trace', '-t', action='store_true',
+                        default=os.environ.get('CLOUDSTACK_TRACE', False),
+                        help='trace the HTTP requests done on stderr')
     parser.add_argument('command', metavar="COMMAND",
                         help='Cloudstack API command to execute')
 
@@ -74,7 +76,7 @@ def main():
                         nargs='*', type=parse_option,
                         help='Cloudstack API argument')
 
-    options = parser.parse_args()
+    options = parser.parse_args(args=args)
     command = options.command
     kwargs = defaultdict(set)
     for arg in options.arguments:
@@ -88,43 +90,38 @@ def main():
 
     theme = config.pop('theme', 'default')
 
+    fetch_result = 'Async' not in command and not getattr(options, 'async')
+
     if options.post:
         config['method'] = 'post'
+    if options.trace:
+        config['trace'] = True
     cs = CloudStack(**config)
     ok = True
+    response = None
+
     try:
-        response = getattr(cs, command)(**kwargs)
+        response = getattr(cs, command)(fetch_result=fetch_result,
+                                        **kwargs)
     except CloudStackException as e:
-        response = e.args[1]
-        if not options.quiet:
-            sys.stderr.write("Cloudstack error: HTTP response "
-                             "{0}\n".format(response.status_code))
+        ok = False
+        if e.response is not None:
+            if not options.quiet:
+                sys.stderr.write("CloudStack error: ")
+                sys.stderr.write("\n".join((str(arg) for arg in e.args)))
+                sys.stderr.write("\n")
 
-        try:
-            response = json.loads(response.text)
-        except ValueError:
-            sys.stderr.write(response.text)
-            sys.stderr.write("\n")
-            sys.exit(1)
-
-    if 'Async' not in command and 'jobid' in response and not getattr(
-            options, 'async'):
-        if not options.quiet:
-            sys.stderr.write("Polling result... ^C to abort\n")
-        while True:
             try:
-                res = cs.queryAsyncJobResult(**response)
-                if res['jobstatus'] != 0:
-                    response = res
-                    if res['jobresultcode'] != 0:
-                        ok = False
-                    break
-                time.sleep(3)
-            except KeyboardInterrupt:
-                if not options.quiet:
-                    sys.stderr.write("Result not ready yet.\n")
-                break
+                response = json.loads(e.response.text)
+            except ValueError:
+                sys.stderr.write(e.response.text)
+                sys.stderr.write("\n")
+        else:
+            message, data = (e.args[0], e.args[0:])
+            sys.stderr.write("Error: {0}\n{1}\n".format(message, data))
 
-    sys.stdout.write(_format_json(response, theme=theme))
-    sys.stdout.write('\n')
-    sys.exit(int(not ok))
+    if response:
+        sys.stdout.write(_format_json(response, theme=theme))
+        sys.stdout.write('\n')
+
+    return not ok
